@@ -117,7 +117,7 @@ DownloadFile(const char *uri, const char *_base)
 {
   assert(Net::DownloadManager::IsAvailable());
 
-  const ACPToWideConverter base(_base);
+  const UTF8ToWideConverter base(_base);
   if (!base.IsValid())
     return tstring();
 
@@ -137,9 +137,8 @@ DownloadFile(const char *uri, const char *_base)
     return tstring();
   }
 
-  TCHAR path[MAX_PATH];
-  LocalPath(path, base);
-  return path;
+  TCHAR buffer[MAX_PATH];
+  return LocalPath(buffer, base);
 }
 
 class DownloadFilePickerWidget final
@@ -265,8 +264,8 @@ DownloadFilePickerWidget::RefreshList()
 
   FileRepository repository;
 
-  TCHAR path[MAX_PATH];
-  LocalPath(path, _T("repository"));
+  TCHAR buffer[MAX_PATH];
+  const auto path = LocalPath(buffer, _T("repository"));
   FileLineReaderA reader(path);
   if (!reader.error()) {
     ParseFileRepository(repository, reader);
