@@ -25,7 +25,7 @@ Copyright_License {
 #include "Compatibility/path.h"
 #include "Util/StringUtil.hpp"
 #include "Util/StringFormat.hpp"
-#include "Util/StringAPI.hpp"
+#include "Util/StringAPI.hxx"
 #include "Asset.hpp"
 
 #include "OS/FileUtil.hpp"
@@ -104,7 +104,7 @@ SetPrimaryDataPath(const TCHAR *path)
   data_path_length = StringLength(data_path);
 }
 
-void
+const TCHAR *
 LocalPath(TCHAR *gcc_restrict buffer, const TCHAR *gcc_restrict file)
 {
   assert(data_path != nullptr);
@@ -112,9 +112,11 @@ LocalPath(TCHAR *gcc_restrict buffer, const TCHAR *gcc_restrict file)
   memcpy(buffer, data_path, data_path_length * sizeof(data_path[0]));
   buffer[data_path_length] = _T(DIR_SEPARATOR);
   _tcscpy(buffer + data_path_length + 1, file);
+
+  return buffer;
 }
 
-TCHAR *
+const TCHAR *
 LocalPath(TCHAR *gcc_restrict buffer, const TCHAR *gcc_restrict subdir,
           const TCHAR *gcc_restrict name)
 {
