@@ -25,7 +25,7 @@ Copyright_License {
 #include "LocalPath.hpp"
 #include "OS/PathName.hpp"
 #include "Compatibility/path.h"
-#include "Util/StringAPI.hpp"
+#include "Util/StringAPI.hxx"
 #include "Util/StringUtil.hpp"
 #include "Util/StringPointer.hxx"
 #include "Util/AllocatedString.hxx"
