@@ -24,7 +24,6 @@ Copyright_License {
 #ifndef XCSOAR_RASTER_BUFFER_HPP
 #define XCSOAR_RASTER_BUFFER_HPP
 
-#include "Util/NonCopyable.hpp"
 #include "Util/AllocatedGrid.hpp"
 #include "Compiler.h"
 
@@ -32,7 +31,7 @@ Copyright_License {
 
 #include <stdint.h>
 
-class RasterBuffer : private NonCopyable {
+class RasterBuffer {
 public:
   /** invalid value for terrain */
   static constexpr short TERRAIN_INVALID = -32768;
@@ -76,6 +75,9 @@ public:
   RasterBuffer(unsigned _width, unsigned _height)
     :data(_width, _height) {}
 
+  RasterBuffer(const RasterBuffer &) = delete;
+  RasterBuffer &operator=(const RasterBuffer &) = delete;
+
   bool IsDefined() const {
     return data.IsDefined();
   }
