@@ -93,102 +93,275 @@ static int jp2_getuint16(jas_stream_t *in, uint_fast16_t *val);
 #endif /* ENABLE_JASPER_IMAGE */
 static int jp2_getuint32(jas_stream_t *in, uint_fast32_t *val);
 static int jp2_getuint64(jas_stream_t *in, uint_fast64_t *val);
-#ifdef JASPER_DISABLED
+
+#ifdef JAS_ENABLE_ENCODER
 static int jp2_putuint8(jas_stream_t *out, uint_fast8_t val);
 static int jp2_putuint16(jas_stream_t *out, uint_fast16_t val);
 static int jp2_putuint32(jas_stream_t *out, uint_fast32_t val);
 static int jp2_putuint64(jas_stream_t *out, uint_fast64_t val);
-#endif /* JASPER_DISABLED */
+#endif
 
 #ifdef ENABLE_JASPER_IMAGE
 static int jp2_getint(jas_stream_t *in, int s, int n, int_fast32_t *val);
 #endif /* ENABLE_JASPER_IMAGE */
 
 #ifdef JASPER_DISABLED
-void jp2_box_dump(jp2_box_t *box, FILE *out);
+static void jp2_box_dump(const jp2_box_t *box, FILE *out);
 #else
 #define jp2_box_dump(box, out)
 #endif /* JASPER_DISABLED */
 
 static int jp2_jp_getdata(jp2_box_t *box, jas_stream_t *in);
-#ifdef JASPER_DISABLED
-static int jp2_jp_putdata(jp2_box_t *box, jas_stream_t *out);
-#endif /* JASPER_DISABLED */
 static int jp2_ftyp_getdata(jp2_box_t *box, jas_stream_t *in);
-#ifdef JASPER_DISABLED
-static int jp2_ftyp_putdata(jp2_box_t *box, jas_stream_t *out);
-#endif /* JASPER_DISABLED */
 #ifdef ENABLE_JASPER_IMAGE
 static int jp2_ihdr_getdata(jp2_box_t *box, jas_stream_t *in);
-static int jp2_ihdr_putdata(jp2_box_t *box, jas_stream_t *out);
 static void jp2_bpcc_destroy(jp2_box_t *box);
 static int jp2_bpcc_getdata(jp2_box_t *box, jas_stream_t *in);
-static int jp2_bpcc_putdata(jp2_box_t *box, jas_stream_t *out);
 static int jp2_colr_getdata(jp2_box_t *box, jas_stream_t *in);
-static int jp2_colr_putdata(jp2_box_t *box, jas_stream_t *out);
+static void jp2_colr_dumpdata(const jp2_box_t *box, FILE *out);
 static void jp2_colr_destroy(jp2_box_t *box);
 static void jp2_cdef_destroy(jp2_box_t *box);
 static int jp2_cdef_getdata(jp2_box_t *box, jas_stream_t *in);
-static int jp2_cdef_putdata(jp2_box_t *box, jas_stream_t *out);
+static void jp2_cdef_dumpdata(const jp2_box_t *box, FILE *out);
 static void jp2_cmap_destroy(jp2_box_t *box);
 static int jp2_cmap_getdata(jp2_box_t *box, jas_stream_t *in);
-static int jp2_cmap_putdata(jp2_box_t *box, jas_stream_t *out);
+static void jp2_cmap_dumpdata(const jp2_box_t *box, FILE *out);
 static void jp2_pclr_destroy(jp2_box_t *box);
 static int jp2_pclr_getdata(jp2_box_t *box, jas_stream_t *in);
-static int jp2_pclr_putdata(jp2_box_t *box, jas_stream_t *out);
-static void jp2_pclr_dumpdata(jp2_box_t *box, FILE *out);
+static void jp2_pclr_dumpdata(const jp2_box_t *box, FILE *out);
 #endif /* ENABLE_JASPER_IMAGE */
 
+#ifdef JAS_ENABLE_ENCODER
+
+static int jp2_jp_putdata(const jp2_box_t *box, jas_stream_t *out);
+static int jp2_ftyp_putdata(const jp2_box_t *box, jas_stream_t *out);
+static int jp2_ihdr_putdata(const jp2_box_t *box, jas_stream_t *out);
+static int jp2_bpcc_putdata(const jp2_box_t *box, jas_stream_t *out);
+static int jp2_colr_putdata(const jp2_box_t *box, jas_stream_t *out);
+static int jp2_cdef_putdata(const jp2_box_t *box, jas_stream_t *out);
+static int jp2_cmap_putdata(const jp2_box_t *box, jas_stream_t *out);
+static int jp2_pclr_putdata(const jp2_box_t *box, jas_stream_t *out);
+
+#endif /* JAS_ENABLE_ENCODER */
+
 /******************************************************************************\
 * Local data.
 \******************************************************************************/
 
 static const jp2_boxinfo_t jp2_boxinfos[] = {
-	{JP2_BOX_JP, "JP", 0,
-	  {0, 0, jp2_jp_getdata}},
-	{JP2_BOX_FTYP, "FTYP", 0,
-	  {0, 0, jp2_ftyp_getdata}},
-	{JP2_BOX_JP2H, "JP2H", JP2_BOX_SUPER,
-	  {0, 0, 0}},
+	{
+		JP2_BOX_JP, 0, "JP",
+		{
+			0, 0,
+			jp2_jp_getdata,
+#ifdef JAS_ENABLE_ENCODER
+			jp2_jp_putdata,
+#endif
+		},
+	},
+	{
+		JP2_BOX_FTYP, 0, "FTYP",
+		{
+			0, 0,
+			jp2_ftyp_getdata,
+#ifdef JAS_ENABLE_ENCODER
+			jp2_ftyp_putdata,
+#endif
+		},
+	},
+	{
+		JP2_BOX_JP2H, JP2_BOX_SUPER, "JP2H",
+		{
+			0, 0,
+			0,
+#ifdef JAS_ENABLE_ENCODER
+			0,
+#endif
+		},
+	},
 #ifdef ENABLE_JASPER_IMAGE
-	{JP2_BOX_IHDR, "IHDR", 0,
-	  {0, 0, jp2_ihdr_getdata}},
-	{JP2_BOX_BPCC, "BPCC", 0,
-	  {0, jp2_bpcc_destroy, jp2_bpcc_getdata}},
-	{JP2_BOX_COLR, "COLR", 0,
-	  {0, jp2_colr_destroy, jp2_colr_getdata}},
-	{JP2_BOX_PCLR, "PCLR", 0,
-	  {0, jp2_pclr_destroy, jp2_pclr_getdata}},
-	{JP2_BOX_CMAP, "CMAP", 0,
-	  {0, jp2_cmap_destroy, jp2_cmap_getdata}},
-	{JP2_BOX_CDEF, "CDEF", 0,
-	  {0, jp2_cdef_destroy, jp2_cdef_getdata, jp2_cdef_putdata, jp2_cdef_dumpdata}},
+	{
+		JP2_BOX_IHDR, 0, "IHDR",
+		{
+			0, 0,
+			jp2_ihdr_getdata,
+#ifdef JAS_ENABLE_ENCODER
+			jp2_ihdr_putdata,
+#endif
+		},
+	},
+	{
+		JP2_BOX_BPCC, 0, "BPCC",
+		{
+			0, jp2_bpcc_destroy,
+			jp2_bpcc_getdata,
+#ifdef JAS_ENABLE_ENCODER
+			jp2_bpcc_putdata,
+#endif
+		},
+	},
+	{
+		JP2_BOX_COLR, 0, "COLR",
+		{
+			0, jp2_colr_destroy,
+			jp2_colr_getdata,
+#ifdef JAS_ENABLE_ENCODER
+			jp2_colr_putdata,
+#endif
+		},
+	},
+	{
+		JP2_BOX_PCLR, 0, "PCLR",
+		{
+			0, jp2_pclr_destroy,
+			jp2_pclr_getdata,
+#ifdef JAS_ENABLE_ENCODER
+			jp2_pclr_putdata,
+#endif
+		},
+	},
+	{
+		JP2_BOX_CMAP, 0, "CMAP",
+		{
+			0, jp2_cmap_destroy,
+			jp2_cmap_getdata,
+#ifdef JAS_ENABLE_ENCODER
+			jp2_cmap_putdata,
+#endif
+		},
+	},
+	{
+		JP2_BOX_CDEF, 0, "CDEF",
+		{
+			0, jp2_cdef_destroy,
+			jp2_cdef_getdata,
+#ifdef JAS_ENABLE_ENCODER
+			jp2_cdef_putdata,
+#endif
+		},
+	},
 #endif /* ENABLE_JASPER_IMAGE */
-	{JP2_BOX_RES, "RES", JP2_BOX_SUPER,
-	  {0, 0, 0}},
-	{JP2_BOX_RESC, "RESC", 0,
-	  {0, 0, 0}},
-	{JP2_BOX_RESD, "RESD", 0,
-	  {0, 0, 0}},
-	{JP2_BOX_JP2C, "JP2C", JP2_BOX_NODATA,
-	  {0, 0, 0}},
-	{JP2_BOX_JP2I, "JP2I", 0,
-	  {0, 0, 0}},
-	{JP2_BOX_XML, "XML", 0,
-	  {0, 0, 0}},
-	{JP2_BOX_UUID, "UUID", 0,
-	  {0, 0, 0}},
-	{JP2_BOX_UINF, "UINF", JP2_BOX_SUPER,
-	  {0, 0, 0}},
-	{JP2_BOX_ULST, "ULST", 0,
-	  {0, 0, 0}},
-	{JP2_BOX_URL, "URL", 0,
-	  {0, 0, 0}},
-	{0, 0, 0, {0, 0, 0}},
+	{
+		JP2_BOX_RES, JP2_BOX_SUPER, "RES",
+		{
+			0, 0,
+			0,
+#ifdef JAS_ENABLE_ENCODER
+			0,
+#endif
+		},
+	},
+	{
+		JP2_BOX_RESC, 0, "RESC",
+		{
+			0, 0,
+			0,
+#ifdef JAS_ENABLE_ENCODER
+			0,
+#endif
+		},
+	},
+	{
+		JP2_BOX_RESD, 0, "RESD",
+		{
+			0, 0,
+			0,
+#ifdef JAS_ENABLE_ENCODER
+			0,
+#endif
+		},
+	},
+	{
+		JP2_BOX_JP2C, JP2_BOX_NODATA, "JP2C",
+		{
+			0, 0,
+			0,
+#ifdef JAS_ENABLE_ENCODER
+			0,
+#endif
+		},
+	},
+	{
+		JP2_BOX_JP2I, 0, "JP2I",
+		{
+			0, 0,
+			0,
+#ifdef JAS_ENABLE_ENCODER
+			0,
+#endif
+		},
+	},
+	{
+		JP2_BOX_XML, 0, "XML",
+		{
+			0, 0,
+			0,
+#ifdef JAS_ENABLE_ENCODER
+			0,
+#endif
+		},
+	},
+	{
+		JP2_BOX_UUID, 0, "UUID",
+		{
+			0, 0,
+			0,
+#ifdef JAS_ENABLE_ENCODER
+			0,
+#endif
+		},
+	},
+	{
+		JP2_BOX_UINF, JP2_BOX_SUPER, "UINF",
+		{
+			0, 0,
+			0,
+#ifdef JAS_ENABLE_ENCODER
+			0,
+#endif
+		},
+	},
+	{
+		JP2_BOX_ULST, 0, "ULST",
+		{
+			0, 0,
+			0,
+#ifdef JAS_ENABLE_ENCODER
+			0,
+#endif
+		},
+	},
+	{
+		JP2_BOX_URL, 0, "URL",
+		{
+			0, 0,
+			0,
+#ifdef JAS_ENABLE_ENCODER
+			0,
+#endif
+		},
+	},
+	{
+		0, 0, 0,
+		{
+			0, 0,
+			0,
+#ifdef JAS_ENABLE_ENCODER
+			0,
+#endif
+		},
+	},
 };
 
 static const jp2_boxinfo_t jp2_boxinfo_unk = {
-	0, "Unknown", 0, {0, 0, 0}
+	0, 0, "Unknown",
+	{
+		0, 0,
+		0,
+#ifdef JAS_ENABLE_ENCODER
+		0,
+#endif
+	}
 };
 
 /******************************************************************************\
@@ -202,34 +375,35 @@ static jp2_box_t *jp2_box_create0(void)
 		return 0;
 	}
 	memset(box, 0, sizeof(jp2_box_t));
-	box->type = 0;
-	box->len = 0;
 	// Mark the box data as never having been constructed
 	// so that we will not errantly attempt to destroy it later.
 	box->ops = &jp2_boxinfo_unk.ops;
 	return box;
 }
 
-#ifdef ENABLE_JASPER_ENCODE
+#ifdef JAS_ENABLE_ENCODER
 
 jp2_box_t *jp2_box_create(int type)
 {
+	const jp2_boxinfo_t *boxinfo = jp2_boxinfolookup(type);
+	assert(boxinfo != NULL);
+	if (boxinfo == &jp2_boxinfo_unk) {
+		// on error, jp2_boxinfolookup() returns &jp2_boxinfo_unk
+		return NULL;
+	}
+
 	jp2_box_t *box;
-	const jp2_boxinfo_t *boxinfo;
 	if (!(box = jp2_box_create0())) {
 		return 0;
 	}
 	box->type = type;
 	box->len = 0;
-	if (!(boxinfo = jp2_boxinfolookup(type))) {
-		return 0;
-	}
 	box->info = boxinfo;
 	box->ops = &boxinfo->ops;
 	return box;
 }
 
-#endif /* ENABLE_JASPER_ENCODE */
+#endif
 
 /******************************************************************************\
 * Box destructor.
@@ -364,10 +538,9 @@ error:
 }
 
 #ifdef JASPER_DISABLED
-void jp2_box_dump(jp2_box_t *box, FILE *out)
+static void jp2_box_dump(const jp2_box_t *box, FILE *out)
 {
-	const jp2_boxinfo_t *boxinfo;
-	boxinfo = jp2_boxinfolookup(box->type);
+	const jp2_boxinfo_t *boxinfo = box->info;
 	assert(boxinfo);
 
 	fprintf(out, "JP2 box: ");
@@ -517,12 +690,12 @@ static int jp2_cdef_getdata(jp2_box_t *box, jas_stream_t *in)
 }
 #endif /* ENABLE_JASPER_IMAGE */
 
-#ifdef JASPER_DISABLED
-
 /******************************************************************************\
 * Box output.
 \******************************************************************************/
 
+#ifdef JAS_ENABLE_ENCODER
+
 int jp2_box_put(jp2_box_t *box, jas_stream_t *out)
 {
 	jas_stream_t *tmpstream;
@@ -582,18 +755,18 @@ error:
 	return -1;
 }
 
-static int jp2_jp_putdata(jp2_box_t *box, jas_stream_t *out)
+static int jp2_jp_putdata(const jp2_box_t *box, jas_stream_t *out)
 {
-	jp2_jp_t *jp = &box->data.jp;
+	const jp2_jp_t *jp = &box->data.jp;
 	if (jp2_putuint32(out, jp->magic)) {
 		return -1;
 	}
 	return 0;
 }
 
-static int jp2_ftyp_putdata(jp2_box_t *box, jas_stream_t *out)
+static int jp2_ftyp_putdata(const jp2_box_t *box, jas_stream_t *out)
 {
-	jp2_ftyp_t *ftyp = &box->data.ftyp;
+	const jp2_ftyp_t *ftyp = &box->data.ftyp;
 	unsigned int i;
 	if (jp2_putuint32(out, ftyp->majver) || jp2_putuint32(out, ftyp->minver)) {
 		return -1;
@@ -606,9 +779,9 @@ static int jp2_ftyp_putdata(jp2_box_t *box, jas_stream_t *out)
 	return 0;
 }
 
-static int jp2_ihdr_putdata(jp2_box_t *box, jas_stream_t *out)
+static int jp2_ihdr_putdata(const jp2_box_t *box, jas_stream_t *out)
 {
-	jp2_ihdr_t *ihdr = &box->data.ihdr;
+	const jp2_ihdr_t *ihdr = &box->data.ihdr;
 	if (jp2_putuint32(out, ihdr->height) || jp2_putuint32(out, ihdr->width) ||
 	  jp2_putuint16(out, ihdr->numcmpts) || jp2_putuint8(out, ihdr->bpc) ||
 	  jp2_putuint8(out, ihdr->comptype) || jp2_putuint8(out, ihdr->csunk) ||
@@ -618,9 +791,9 @@ static int jp2_ihdr_putdata(jp2_box_t *box, jas_stream_t *out)
 	return 0;
 }
 
-static int jp2_bpcc_putdata(jp2_box_t *box, jas_stream_t *out)
+static int jp2_bpcc_putdata(const jp2_box_t *box, jas_stream_t *out)
 {
-	jp2_bpcc_t *bpcc = &box->data.bpcc;
+	const jp2_bpcc_t *bpcc = &box->data.bpcc;
 	unsigned int i;
 	for (i = 0; i < bpcc->numcmpts; ++i) {
 		if (jp2_putuint8(out, bpcc->bpcs[i])) {
@@ -630,9 +803,9 @@ static int jp2_bpcc_putdata(jp2_box_t *box, jas_stream_t *out)
 	return 0;
 }
 
-static int jp2_colr_putdata(jp2_box_t *box, jas_stream_t *out)
+static int jp2_colr_putdata(const jp2_box_t *box, jas_stream_t *out)
 {
-	jp2_colr_t *colr = &box->data.colr;
+	const jp2_colr_t *colr = &box->data.colr;
 	if (jp2_putuint8(out, colr->method) || jp2_putuint8(out, colr->pri) ||
 	  jp2_putuint8(out, colr->approx)) {
 		return -1;
@@ -644,26 +817,24 @@ static int jp2_colr_putdata(jp2_box_t *box, jas_stream_t *out)
 		}
 		break;
 	case JP2_COLR_ICC:
-		if (jas_stream_write(out, colr->iccp,
-		  JAS_CAST(int, colr->iccplen)) != JAS_CAST(int, colr->iccplen))
+		if (jas_stream_write(out, colr->iccp, colr->iccplen) != colr->iccplen)
 			return -1;
 		break;
 	}
 	return 0;
 }
 
-static int jp2_cdef_putdata(jp2_box_t *box, jas_stream_t *out)
+static int jp2_cdef_putdata(const jp2_box_t *box, jas_stream_t *out)
 {
-	jp2_cdef_t *cdef = &box->data.cdef;
+	const jp2_cdef_t *cdef = &box->data.cdef;
 	unsigned int i;
-	jp2_cdefchan_t *ent;
 
 	if (jp2_putuint16(out, cdef->numchans)) {
 		return -1;
 	}
 
 	for (i = 0; i < cdef->numchans; ++i) {
-		ent = &cdef->ents[i];
+		const jp2_cdefchan_t *ent = &cdef->ents[i];
 		if (jp2_putuint16(out, ent->channo) ||
 		  jp2_putuint16(out, ent->type) ||
 		  jp2_putuint16(out, ent->assoc)) {
@@ -673,7 +844,7 @@ static int jp2_cdef_putdata(jp2_box_t *box, jas_stream_t *out)
 	return 0;
 }
 
-#endif /* JASPER_DISABLED */
+#endif /* JAS_ENABLE_ENCODER */
 
 /******************************************************************************\
 * Input operations for primitive types.
@@ -694,46 +865,21 @@ static int jp2_getuint8(jas_stream_t *in, uint_fast8_t *val)
 
 static int jp2_getuint16(jas_stream_t *in, uint_fast16_t *val)
 {
-	uint_fast16_t v;
-	int c;
-	if ((c = jas_stream_getc(in)) == EOF) {
+	jas_uchar buffer[2];
+	if (jas_stream_read(in, buffer, sizeof(buffer)) != sizeof(buffer))
 		return -1;
-	}
-	v = c;
-	if ((c = jas_stream_getc(in)) == EOF) {
-		return -1;
-	}
-	v = (v << 8) | c;
-	if (val) {
-		*val = v;
-	}
+	*val = (uint_fast16_t)buffer[0] << 8 | (uint_fast16_t)buffer[1];
 	return 0;
 }
 #endif /* ENABLE_JASPER_IMAGE */
 
 static int jp2_getuint32(jas_stream_t *in, uint_fast32_t *val)
 {
-	uint_fast32_t v;
-	int c;
-	if ((c = jas_stream_getc(in)) == EOF) {
+	jas_uchar buffer[4];
+	if (jas_stream_read(in, buffer, sizeof(buffer)) != sizeof(buffer))
 		return -1;
-	}
-	v = c;
-	if ((c = jas_stream_getc(in)) == EOF) {
-		return -1;
-	}
-	v = (v << 8) | c;
-	if ((c = jas_stream_getc(in)) == EOF) {
-		return -1;
-	}
-	v = (v << 8) | c;
-	if ((c = jas_stream_getc(in)) == EOF) {
-		return -1;
-	}
-	v = (v << 8) | c;
-	if (val) {
-		*val = v;
-	}
+	*val = (uint_fast32_t)buffer[0] << 24 | (uint_fast32_t)buffer[1] << 16
+		| (uint_fast32_t)buffer[2] << 8 | (uint_fast32_t)buffer[3];
 	return 0;
 }
 
@@ -756,12 +902,12 @@ static int jp2_getuint64(jas_stream_t *in, uint_fast64_t *val)
 	return 0;
 }
 
-#ifdef JASPER_DISABLED
-
 /******************************************************************************\
 * Output operations for primitive types.
 \******************************************************************************/
 
+#ifdef JAS_ENABLE_ENCODER
+
 static int jp2_putuint8(jas_stream_t *out, uint_fast8_t val)
 {
 	if (jas_stream_putc(out, val & 0xff) == EOF) {
@@ -799,7 +945,7 @@ static int jp2_putuint64(jas_stream_t *out, uint_fast64_t val)
 	return 0;
 }
 
-#endif /* JASPER_DISABLED */
+#endif
 
 /******************************************************************************\
 * Miscellaneous code.
@@ -858,8 +1004,9 @@ static int jp2_cmap_getdata(jp2_box_t *box, jas_stream_t *in)
 	return 0;
 }
 
-#ifdef JASPER_DISABLED
-static int jp2_cmap_putdata(jp2_box_t *box, jas_stream_t *out)
+#ifdef JAS_ENABLE_ENCODER
+
+static int jp2_cmap_putdata(const jp2_box_t *box, jas_stream_t *out)
 {
 	/* Eliminate compiler warning about unused variables. */
 	(void)box;
@@ -867,7 +1014,8 @@ static int jp2_cmap_putdata(jp2_box_t *box, jas_stream_t *out)
 
 	return -1;
 }
-#endif /* JASPER_DISABLED */
+
+#endif
 
 static void jp2_pclr_destroy(jp2_box_t *box)
 {
@@ -924,22 +1072,24 @@ static int jp2_pclr_getdata(jp2_box_t *box, jas_stream_t *in)
 	return 0;
 }
 
-#ifdef JASPER_DISABLED
-static int jp2_pclr_putdata(jp2_box_t *box, jas_stream_t *out)
+#ifdef JAS_ENABLE_ENCODER
+
+static int jp2_pclr_putdata(const jp2_box_t *box, jas_stream_t *out)
 {
 #if 0
-	jp2_pclr_t *pclr = &box->data.pclr;
+	const jp2_pclr_t *pclr = &box->data.pclr;
 #endif
 	/* Eliminate warning about unused variable. */
 	(void)box;
 	(void)out;
 	return -1;
 }
-#endif /* JASPER_DISABLED */
 
-static void jp2_pclr_dumpdata(jp2_box_t *box, FILE *out)
+#endif
+
+static void jp2_pclr_dumpdata(const jp2_box_t *box, FILE *out)
 {
-	jp2_pclr_t *pclr = &box->data.pclr;
+	const jp2_pclr_t *pclr = &box->data.pclr;
 	unsigned int i;
 	int j;
 	fprintf(out, "numents=%d; numchans=%d\n", (int) pclr->numlutents,
